<?php

namespace Bixo\Order\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Order\Forms\Order as OrderForm;
use Bixo\Order\Http\Requests\OrderResourceRequest;
use Bixo\Order\Http\Resources\OrderResource;
use Bixo\Order\Http\Resources\OrdersCollection;
use Bixo\Order\Models\Order;
use Bixo\Order\Actions\OrderAction;
use Bixo\Order\Actions\OrderActions;


/**
 * Resource controller class for order.
 */
class OrderResourceController extends BaseController
{

    /**
     * Initialize order resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = OrderForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.order.modules'), 'order', guard_url('order'));
            return $next($request);
        });
    }

    /**
     * Display a list of order.
     *
     * @return Response
     */
    public function index(OrderResourceRequest $request)
    {
        $request = $request->all();
        $page = OrderActions::run('paginate', $request);

        $data = new OrdersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('order::order.names'))
            ->view('order::order.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display order.
     *
     * @param Request $request
     * @param Model   $order
     *
     * @return Response
     */
    public function show(OrderResourceRequest $request, Order $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OrderResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('order::order.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('order::order.show')
            ->output();
    }

    /**
     * Show the form for creating a new order.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(OrderResourceRequest $request, Order $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OrderResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('order::order.name'))
            ->view('order::order.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new order.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(OrderResourceRequest $request, Order $model)
    {
        try {
            $request = $request->all();
            $model = OrderAction::run('store', $model, $request);
            $data = new OrderResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('order::order.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('order/order/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/order/order'))
                ->redirect();
        }

    }

    /**
     * Show order for editing.
     *
     * @param Request $request
     * @param Model   $order
     *
     * @return Response
     */
    public function edit(OrderResourceRequest $request, Order $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OrderResource($model);
        // return view('order::order.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('order::order.name'))
            ->view('order::order.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the order.
     *
     * @param Request $request
     * @param Model   $order
     *
     * @return Response
     */
    public function update(OrderResourceRequest $request, Order $model)
    {
        try {
            $request = $request->all();
            $model = OrderAction::run('update', $model, $request);
            $data = new OrderResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('order::order.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('order/order/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('order/order/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the order.
     *
     * @param Model   $order
     *
     * @return Response
     */
    public function destroy(OrderResourceRequest $request, Order $model)
    {
        try {

            $request = $request->all();
            $model = OrderAction::run('destroy', $model, $request);
            $data = new OrderResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('order::order.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('order/order/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('order/order/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
